@@ -16,6 +16,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+source ../tests/functions.sh
 set -e
+set -x
 
-$VG guestfish --ro --format=raw -a ../test-data/phony-guests/fedora.img run
+skip_if_skipped
+skip_unless_phony_guest fedora.img
+
+$VG guestfish \
+    --ro --format=raw -a $top_builddir/test-data/phony-guests/fedora.img \
+    run
