@@ -18,12 +18,12 @@
 
 # Test if we can handle qemu death synchronously.
 
+source ./functions.sh
 set -e
+set -x
 
-if [ "$(guestfish get-backend)" != "direct" ]; then
-    echo "$0: test skipped because default backend is not 'direct'"
-    exit 77
-fi
+skip_if_skipped
+skip_unless_backend direct
 
 rm -f qemudie-synch.pid
 
